@@ -1,33 +1,171 @@
-from rx.core.typing import Observable
-from PIL import Image
-import numpy as np
 from abc import ABC, abstractmethod
-from typing import Coroutine
-
 from typing import (
-    Dict,
     List,
     Literal,
     Optional,
-    Tuple,
-    TypedDict,
     Union,
     Any,
     Protocol,
-    runtime_checkable,
-    Callable,
     overload,
-    TypeVar,
+    runtime_checkable,
     AsyncGenerator,
+    TypedDict,
+    Tuple,
+    Dict,
+    Callable,
+    TypeVar,
+    Coroutine,
+    Awaitable,
 )
 
+from rx.core.typing import Observable
+from PIL import Image
+import numpy as np
+
 Callback = Union[
     Callable[..., Any],
     Callable[..., Coroutine[Any, Any, Any]],
 ]
 
 
-# database types
+# schema
+class JsonBaseSchema(TypedDict):
+    type: Any
+    key: Optional[str]
+    title: Optional[str]
+    description: Optional[str]
+    required: Optional[bool]
+    readonly: Optional[bool]
+    placeholder: Optional[str]
+    hidden: Optional[bool]
+    group: Optional[str]
+    defaultValue: Optional[Union[str, int, bool]]
+
+
+class PluginJsonBaseSchema(JsonBaseSchema):
+    store: Optional[bool]
+    onGet: Any
+    onSet: Any
+
+
+class PluginJsonSchemaString(PluginJsonBaseSchema):
+    type: str
+    format: Optional[str]
+    minLength: Optional[int]
+    maxLength: Optional[int]
+
+
+class PluginJsonSchemaNumber(PluginJsonBaseSchema):
+    type: int
+    minimum: Optional[int]
+    maximum: Optional[int]
+
+
+class PluginJsonSchemaBoolean(PluginJsonBaseSchema):
+    type: bool
+
+
+class PluginJsonSchemaEnum(PluginJsonBaseSchema):
+    type: str
+    enum: List[str]
+    multiple: Optional[bool]
+
+
+class PluginJsonSchemaObject(PluginJsonBaseSchema):
+    type: Dict[str, Any]
+    opened: Optional[bool]
+    properties: Optional[Dict[str, "PluginJsonSchema"]]
+
+
+PluginJsonSchema = Union[
+    PluginJsonSchemaString,
+    PluginJsonSchemaNumber,
+    PluginJsonSchemaBoolean,
+    PluginJsonSchemaEnum,
+    PluginJsonSchemaObject,
+]
+
+
+PluginJsonSchemaForm = Dict[str, PluginJsonSchema]
+
+
+class PluginRootSchema(TypedDict):
+    schema: PluginJsonSchemaForm
+
+
+class JsonSchemaString(JsonBaseSchema):
+    type: str
+    format: Optional[str]
+    minLength: Optional[int]
+    maxLength: Optional[int]
+
+
+class JsonSchemaNumber(JsonBaseSchema):
+    type: int
+    minimum: Optional[int]
+    maximum: Optional[int]
+
+
+class JsonSchemaBoolean(JsonBaseSchema):
+    type: bool
+
+
+class JsonSchemaEnum(JsonBaseSchema):
+    type: str
+    enum: List[str]
+    multiple: Optional[bool]
+
+
+class JsonSchemaObject(JsonBaseSchema):
+    type: Dict[str, Any]
+    opened: Optional[bool]
+    properties: Optional[Dict[str, "JsonSchema"]]
+
+
+class JsonSchemaButton(TypedDict):
+    label: str
+    onSubmit: str
+
+
+class JsonSchemaObjectWithButtons(JsonSchemaObject):
+    buttons: Optional[Tuple[JsonSchemaButton, Optional[JsonSchemaButton]]]
+
+
+class JsonSchemaAnyOf(TypedDict):
+    anyOf: List["JsonSchema"]
+
+
+class JsonSchemaArray(JsonBaseSchema):
+    type: List[Any]
+    opened: Optional[bool]
+    items: Optional[Union["JsonSchema", JsonSchemaAnyOf]]
+
+
+JsonSchema = Union[
+    JsonSchemaString,
+    JsonSchemaNumber,
+    JsonSchemaBoolean,
+    JsonSchemaEnum,
+    JsonSchemaObject,
+    JsonSchemaObjectWithButtons,
+    JsonSchemaArray,
+]
+
+
+class JsonSchemaForm(TypedDict):
+    pass
+
+
+class RootSchema(TypedDict):
+    schema: JsonSchemaForm
+
+
+class SchemaConfig(TypedDict):
+    rootSchema: PluginRootSchema
+    config: Dict[str, Union[str, int, bool]]
+
+
+# camera object
 class Go2RtcFFMPEGSource(TypedDict):
     aac: str
     opus: str
@@ -74,6 +212,16 @@ class CameraInput(TypedDict):
 
 CameraRoles = Literal["detect", "record", "stream", "snapshot", "none"]
 
+CameraExtension = Literal[
+    "hub",
+    "prebuffer",
+    "motionDetection",
+    "objectDetection",
+    "audioDetection",
+    "ptz",
+    "intercom",
+]
+
 
 class CameraZone(TypedDict):
     name: str
@@ -129,7 +277,7 @@ class Camera(BaseCamera):
     sources: List[CameraInput]
 
 
-# camera types
+# camera device
 class BaseState(TypedDict):
     timestamp: int
     lastTriggered: int
@@ -264,12 +412,6 @@ StateNames = Literal[
 OnChangeCallback = Callable[[T, T], None]
 
 
-class CameraSource(Protocol):
-    async def is_prebuffering(self, type: str) -> bool: ...
-    async def get_prebuffer_raw_url(self, type: str) -> Optional[str]: ...
-    async def get_stream_info(self) -> Optional[Dict[str, Any]]: ...
-
-
 class CameraDelegate(ABC):
     @abstractmethod
     async def snapshot_request(self, request_id: str) -> bytes:
@@ -306,337 +448,10 @@ class CameraPrebufferDelegate(ABC):
         pass
 
 
-CameraExtension = Literal[
-    "hub",
-    "prebuffer",
-    "motionDetection",
-    "objectDetection",
-    "audioDetection",
-    "ptz",
-    "intercom",
-]
-
-
-class BaseCameraConfig(TypedDict):
-    name: str
-    nativeId: Optional[str]
-    isCloud: Optional[bool]
-    hasLight: Optional[bool]
-    hasSiren: Optional[bool]
-    hasBinarySensor: Optional[bool]
-    hasBattery: Optional[bool]
-    disabled: Optional[bool]
-    info: Optional[CameraInformation]
-
-
-class CameraInputSettings(TypedDict):
-    _id: str
-    name: str
-    roles: List[CameraRoles]
-    urls: List[str]
-
-
-class CameraConfigWithSources(BaseCameraConfig):
-    sources: Optional[List[CameraInputSettings]]
-
-
-class CameraConfigWithDelegate(BaseCameraConfig):
-    delegate: Optional[CameraDelegate]
-
-
-CameraConfig = Union[CameraConfigWithSources, CameraConfigWithDelegate]
-
-
-# plugins types
-PluginConfig = Dict[str, Union[str, int, bool]]
-
-
-class JsonBaseSchema(TypedDict):
-    type: Any
-    key: Optional[str]
-    title: Optional[str]
-    description: Optional[str]
-    required: Optional[bool]
-    readonly: Optional[bool]
-    placeholder: Optional[str]
-    hidden: Optional[bool]
-    group: Optional[str]
-    defaultValue: Optional[Union[str, int, bool]]
-
-
-class PluginJsonBaseSchema(JsonBaseSchema):
-    store: Optional[bool]
-    onGet: Any
-    onSet: Any
-
-
-class PluginJsonSchemaString(PluginJsonBaseSchema):
-    type: str
-    format: Optional[str]
-    minLength: Optional[int]
-    maxLength: Optional[int]
-
-
-class PluginJsonSchemaNumber(PluginJsonBaseSchema):
-    type: int
-    minimum: Optional[int]
-    maximum: Optional[int]
-
-
-class PluginJsonSchemaBoolean(PluginJsonBaseSchema):
-    type: bool
-
-
-class PluginJsonSchemaEnum(PluginJsonBaseSchema):
-    type: str
-    enum: List[str]
-    multiple: Optional[bool]
-
-
-class PluginJsonSchemaObject(PluginJsonBaseSchema):
-    type: Dict[str, Any]
-    opened: Optional[bool]
-    properties: Optional[Dict[str, "PluginJsonSchema"]]
-
-
-PluginJsonSchema = Union[
-    PluginJsonSchemaString,
-    PluginJsonSchemaNumber,
-    PluginJsonSchemaBoolean,
-    PluginJsonSchemaEnum,
-    PluginJsonSchemaObject,
-]
-
-
-PluginJsonSchemaForm = Dict[str, PluginJsonSchema]
-
-
-class PluginRootSchema(TypedDict):
-    schema: PluginJsonSchemaForm
-
-
-# json schema types
-class JsonSchemaString(JsonBaseSchema):
-    type: str
-    format: Optional[str]
-    minLength: Optional[int]
-    maxLength: Optional[int]
-
-
-class JsonSchemaNumber(JsonBaseSchema):
-    type: int
-    minimum: Optional[int]
-    maximum: Optional[int]
-
-
-class JsonSchemaBoolean(JsonBaseSchema):
-    type: bool
-
-
-class JsonSchemaEnum(JsonBaseSchema):
-    type: str
-    enum: List[str]
-    multiple: Optional[bool]
-
-
-class JsonSchemaObject(JsonBaseSchema):
-    type: Dict[str, Any]
-    opened: Optional[bool]
-    properties: Optional[Dict[str, "JsonSchema"]]
-
-
-class JsonSchemaButton(TypedDict):
-    label: str
-    onSubmit: str
-
-
-class JsonSchemaObjectWithButtons(JsonSchemaObject):
-    buttons: Optional[Tuple[JsonSchemaButton, Optional[JsonSchemaButton]]]
-
-
-class JsonSchemaAnyOf(TypedDict):
-    anyOf: List["JsonSchema"]
-
-
-class JsonSchemaArray(JsonBaseSchema):
-    type: List[Any]
-    opened: Optional[bool]
-    items: Optional[Union["JsonSchema", JsonSchemaAnyOf]]
-
-
-JsonSchema = Union[
-    JsonSchemaString,
-    JsonSchemaNumber,
-    JsonSchemaBoolean,
-    JsonSchemaEnum,
-    JsonSchemaObject,
-    JsonSchemaObjectWithButtons,
-    JsonSchemaArray,
-]
-
-
-class JsonSchemaForm(TypedDict):
-    pass
-
-
-class RootSchema(TypedDict):
-    schema: JsonSchemaForm
-
-
-class SchemaConfig(TypedDict):
-    rootSchema: PluginRootSchema
-    config: Dict[str, Union[str, int, bool]]
-
-
-class IceServer(TypedDict):
-    urls: List[str]
-    username: Optional[str]
-    credential: Optional[str]
-
-
-class FrameMetadata(TypedDict):
-    format: str
-    frameSize: int
-    width: int
-    height: int
-
-
-class Frame(TypedDict):
-    metadata: FrameMetadata
-    data: bytes
-
-
-# camera storage
-@runtime_checkable
-class CameraStorage(Protocol):
-    async def initialize_storage(self) -> None: ...
-    async def get_value(self, path: str, default_value: Any = None) -> Any: ...
-    async def set_value(self, path: str, new_value: Any) -> None: ...
-    def has_value(self, path: str) -> bool: ...
-    async def get_config(self) -> SchemaConfig: ...
-    async def set_config(self, new_config: Dict[str, Any]) -> None: ...
-    async def add_schema(
-        self,
-        schema_or_path: Union[PluginJsonSchemaForm, str],
-        schema: Optional[PluginJsonSchema] = None,
-    ) -> None: ...
-    def remove_schema(self, path: str) -> None: ...
-    async def change_schema(self, path: str, new_schema: Dict[str, Any]) -> None: ...
-    def get_schema(self, path: str) -> Optional[PluginJsonSchema]: ...
-    def has_schema(self, path: str) -> bool: ...
-    async def _resolve_on_get_functions(
-        self,
-        schema: Union[PluginJsonSchemaForm, PluginJsonSchema],
-        base_schema_path: str = "",
-    ) -> None: ...
-    async def _resolve_on_get_functions_for_object(
-        self,
-        schema: PluginJsonSchemaForm,
-        base_schema_path: str,
-    ) -> None: ...
-    async def _resolve_on_get_functions_for_schema(self, schema_path: str) -> None: ...
-    async def _trigger_on_set_for_changes(
-        self,
-        old_config: Dict[str, Any],
-        new_config: Dict[str, Any],
-        path: str = "",
-    ) -> None: ...
-    def _filter_storable_values(
-        self,
-        schema: PluginJsonSchemaForm,
-        base_schema_path: str = "",
-        result: Dict[str, Any] = {},
-    ) -> Dict[str, Any]: ...
-    def _contains_storable_schema(
-        self,
-        schema: Union[PluginJsonSchemaForm, PluginJsonSchema],
-    ) -> bool: ...
-    def _save_db(self) -> None: ...
-    def _remove_db(self) -> None: ...
-
-
-# storage controller
-@runtime_checkable
-class StorageController(Protocol):
-    async def create_camera_storage(
-        self,
-        instance: Any,
-        camera_id: str,
-        schema: Optional[PluginJsonSchemaForm] = None,
-    ) -> CameraStorage: ...
-    def get_camera_storage(self, camera_id: str) -> Optional[CameraStorage]: ...
-    def remove_camera_storage(self, camera_id: str) -> None: ...
-
-
-# logger
-@runtime_checkable
-class PluginLogger(Protocol):
-    def log(self, *args: Any) -> None: ...
-    def error(self, *args: Any) -> None: ...
-    def warn(self, *args: Any) -> None: ...
-    def attention(self, *args: Any) -> None: ...
-    def debug(self, *args: Any) -> None: ...
-    def trace(self, *args: Any) -> None: ...
-
-
-# config service
-@runtime_checkable
-class PluginConfigService(Protocol):
-    def get(
-        self,
-        key: str,
-        default: Optional[Any] = None,
-        validate: Optional[Callable[[Any], bool]] = None,
-        refresh: bool = False,
-        write_if_not_valid: bool = False,
-    ) -> Any: ...
-
-    def has(self, key: str, refresh: bool = False) -> bool: ...
-    def ensure_exists(
-        self, key: str, default: Optional[Any] = None, write: bool = False
-    ) -> None: ...
-    def set(self, key: str, value: Any, write: bool = False) -> None: ...
-    def insert(
-        self, key: str, value: Any, index: Optional[int] = None, write: bool = False
-    ) -> None: ...
-    def push(self, key: str, *values: Any, write: bool = False) -> None: ...
-    def delete(self, key: str, write: bool = False) -> None: ...
-    def all(self, refresh: bool = False) -> Dict[str, Any]: ...
-    def replace(self, new_config: Dict[str, Any], write: bool = False) -> None: ...
-    def update_value(
-        self,
-        path: str,
-        search_key: str,
-        search_value: Any,
-        target_key: str,
-        new_value: Any,
-        write: bool = False,
-    ) -> None: ...
-    def replace_or_add_item(
-        self,
-        path: str,
-        search_key: str,
-        search_value: Any,
-        new_item: Any,
-        write: bool = False,
-    ) -> None: ...
-
-
-# manager todo: improve typing
-@runtime_checkable
-class SystemManager(Protocol):
-    def on(self, event: str, listener: Callback) -> "SystemManager": ...
-    def once(self, event: str, listener: Callback) -> "SystemManager": ...
-    def remove_listener(self, event: str, listener: Callback) -> "SystemManager": ...
-    def remove_all_listeners(self, event: str) -> "SystemManager": ...
-
-
-@runtime_checkable
-class PluginsManager(Protocol):  # todo: add methods
-    def on(self, event: str, listener: Callback) -> "PluginsManager": ...
-    def once(self, event: str, listener: Callback) -> "PluginsManager": ...
-    def remove_listener(self, event: str, listener: Callback) -> "PluginsManager": ...
-    def remove_all_listeners(self, event: str) -> "PluginsManager": ...
+class CameraSource(Protocol):
+    async def is_prebuffering(self, type: str) -> bool: ...
+    async def get_prebuffer_raw_url(self, type: str) -> Optional[str]: ...
+    async def get_stream_info(self) -> Optional[Dict[str, Any]]: ...
 
 
 class ImageCrop(TypedDict):
@@ -659,7 +474,24 @@ class ImageOptions(TypedDict, total=False):
     rotate: int
 
 
-@runtime_checkable
+class IceServer(TypedDict):
+    urls: List[str]
+    username: Optional[str]
+    credential: Optional[str]
+
+
+class FrameMetadata(TypedDict):
+    format: str
+    frameSize: int
+    width: int
+    height: int
+
+
+class Frame(TypedDict):
+    metadata: FrameMetadata
+    data: bytes
+
+
 class VideoFrame(Protocol):
     metadata: FrameMetadata
     image: Image.Image
@@ -674,7 +506,6 @@ class VideoFrame(Protocol):
     def to_image(self, options: Optional[ImageOptions] = None) -> Image.Image: ...
 
 
-@runtime_checkable
 class CameraDevice(Protocol):
     id: str
     plugin_id: str
@@ -696,6 +527,8 @@ class CameraDevice(Protocol):
     has_audio_detector: bool
     has_object_detector: bool
     has_ptz: bool
+    has_prebuffer: bool
+    has_intercom: bool
 
     on_connected: Observable[bool]
     on_light_switched: Observable[LightState]
@@ -721,10 +554,10 @@ class CameraDevice(Protocol):
     async def get_ffmpeg_path(self) -> str: ...
     async def get_ice_servers(self) -> List[IceServer]: ...
     # async def create_session(self, options: Any) -> Any: ... # todo
-    async def stream_video(self, source_name: str, options: Any) -> None: ...
-    async def record_to_file(
-        self, source_name: str, output_path: str, duration: Optional[int] = 60
-    ) -> None: ...
+    # async def stream_video(self, source_name: str, options: Any) -> None: ...  # todo
+    # async def record_to_file(
+    #     self, source_name: str, output_path: str, duration: Optional[int] = 60  # todo
+    # ) -> None: ...
     async def get_value(
         self, state_name: str
     ) -> Union[
@@ -791,13 +624,38 @@ class CameraDevice(Protocol):
     def remove_all_listeners(self) -> None: ...
 
 
-# device manager
-DeviceManagerEventType = Literal[
-    "cameraSelected",
-    "cameraDeselected",
-    "cameraUpdated",
-]
+# camera create config
+class BaseCameraConfig(TypedDict):
+    name: str
+    nativeId: Optional[str]
+    isCloud: Optional[bool]
+    hasLight: Optional[bool]
+    hasSiren: Optional[bool]
+    hasBinarySensor: Optional[bool]
+    hasBattery: Optional[bool]
+    disabled: Optional[bool]
+    info: Optional[CameraInformation]
 
+
+class CameraInputSettings(TypedDict):
+    _id: str
+    name: str
+    roles: List[CameraRoles]
+    urls: List[str]
+
+
+class CameraConfigWithSources(BaseCameraConfig):
+    sources: Optional[List[CameraInputSettings]]
+
+
+class CameraConfigWithDelegate(BaseCameraConfig):
+    delegate: Optional[CameraDelegate]
+
+
+CameraConfig = Union[CameraConfigWithSources, CameraConfigWithDelegate]
+
+
+# events and callbacks
 CameraSelectedCallback = Union[
     Callable[[CameraDevice, CameraExtension], None],
     Callable[[CameraDevice, CameraExtension], Coroutine[None, None, None]],
@@ -819,6 +677,17 @@ CameraUpdatedCallback = Union[
 ]
 
 
+# abstract
+
+
+# DeviceManager
+DeviceManagerEventType = Literal[
+    "cameraSelected",
+    "cameraDeselected",
+    "cameraUpdated",
+]
+
+
 @runtime_checkable
 class DeviceManager(Protocol):
     async def get_camera_by_id(self, id: str) -> Optional[CameraDevice]: ...
@@ -871,7 +740,116 @@ class DeviceManager(Protocol):
     ) -> "DeviceManager": ...
 
 
-# plugin api
+# SystemManager
+@runtime_checkable
+class SystemManager(Protocol):
+    def on(self, event: str, listener: Callback) -> "SystemManager": ...
+    def once(self, event: str, listener: Callback) -> "SystemManager": ...
+    def remove_listener(self, event: str, listener: Callback) -> "SystemManager": ...
+    def remove_all_listeners(self, event: str) -> "SystemManager": ...
+
+
+# PluginsManager
+@runtime_checkable
+class PluginsManager(Protocol):  # todo: add methods
+    def on(self, event: str, listener: Callback) -> "PluginsManager": ...
+    def once(self, event: str, listener: Callback) -> "PluginsManager": ...
+    def remove_listener(self, event: str, listener: Callback) -> "PluginsManager": ...
+    def remove_all_listeners(self, event: str) -> "PluginsManager": ...
+
+
+# camera storage
+P = TypeVar("P")
+
+
+@runtime_checkable
+class CameraStorage(Protocol):
+    async def initialize_storage(self) -> None: ...
+    @overload
+    def get_value(self, path: str) -> Union[Awaitable[P], P, None]: ...
+    @overload
+    def get_value(self, path: str, default_value: P) -> Union[Awaitable[P], P]: ...
+    def get_value(
+        self, path: str, default_value: Optional[P] = None
+    ) -> Union[Awaitable[P], P, None]: ...
+    async def set_value(self, path: str, new_value: Any) -> None: ...
+    def has_value(self, path: str) -> bool: ...
+    async def get_config(self) -> SchemaConfig: ...
+    async def set_config(self, new_config: Dict[str, Any]) -> None: ...
+    @overload
+    async def add_schema(self, schema_or_path: PluginJsonSchemaForm) -> None: ...
+    @overload
+    async def add_schema(
+        self, schema_or_path: str, schema: PluginJsonSchema
+    ) -> None: ...
+    async def add_schema(
+        self,
+        schema_or_path: Union[PluginJsonSchemaForm, str],
+        schema: Optional[PluginJsonSchema] = None,
+    ) -> None: ...
+    def remove_schema(self, path: str) -> None: ...
+    async def change_schema(self, path: str, new_schema: Dict[str, Any]) -> None: ...
+    def get_schema(self, path: str) -> Optional[PluginJsonSchema]: ...
+    def has_schema(self, path: str) -> bool: ...
+
+
+# StorageController
+@runtime_checkable
+class StorageController(Protocol):
+    async def create_camera_storage(
+        self,
+        instance: Any,
+        camera_id: str,
+        schema: Optional[PluginJsonSchemaForm] = None,
+    ) -> CameraStorage: ...
+    def get_camera_storage(self, camera_id: str) -> Optional[CameraStorage]: ...
+    def remove_camera_storage(self, camera_id: str) -> None: ...
+
+
+# config service
+@runtime_checkable
+class PluginConfigService(Protocol):
+    def get(
+        self,
+        key: str,
+        default: Optional[Any] = None,
+        validate: Optional[Callable[[Any], bool]] = None,
+        refresh: bool = False,
+        write_if_not_valid: bool = False,
+    ) -> Any: ...
+
+    def has(self, key: str, refresh: bool = False) -> bool: ...
+    def ensure_exists(
+        self, key: str, default: Optional[Any] = None, write: bool = False
+    ) -> None: ...
+    def set(self, key: str, value: Any, write: bool = False) -> None: ...
+    def insert(
+        self, key: str, value: Any, index: Optional[int] = None, write: bool = False
+    ) -> None: ...
+    def push(self, key: str, *values: Any, write: bool = False) -> None: ...
+    def delete(self, key: str, write: bool = False) -> None: ...
+    def all(self, refresh: bool = False) -> Dict[str, Any]: ...
+    def replace(self, new_config: Dict[str, Any], write: bool = False) -> None: ...
+    def update_value(
+        self,
+        path: str,
+        search_key: str,
+        search_value: Any,
+        target_key: str,
+        new_value: Any,
+        write: bool = False,
+    ) -> None: ...
+    def replace_or_add_item(
+        self,
+        path: str,
+        search_key: str,
+        search_value: Any,
+        new_item: Any,
+        write: bool = False,
+    ) -> None: ...
+
+
+# Plugin API
 APIEventType = Literal["finishLaunching", "shutdown"]
 
 
@@ -895,7 +873,24 @@ class PluginAPI(Protocol):
     ) -> "PluginAPI": ...
 
 
-# base plugin
+# logger
+@runtime_checkable
+class PluginLogger(Protocol):
+    prefix: str
+
+    def log(self, *args: Any) -> None: ...
+    def error(self, *args: Any) -> None: ...
+    def warn(self, *args: Any) -> None: ...
+    def attention(self, *args: Any) -> None: ...
+    def debug(self, *args: Any) -> None: ...
+    def trace(self, *args: Any) -> None: ...
+
+
+# plugin
+PluginConfig = Dict[str, Any]
+JSONValue = Union[str, int, float, bool, Dict[str, Any], List[Any]]
+
+
 class BasePlugin(ABC):
     @abstractmethod
     def __init__(self, logger: PluginLogger, api_instance: PluginAPI) -> None: ...
